@@ -9,7 +9,7 @@
 // movement keys
 {'s', CMD_WAIT},
 {CK_CLEAR, CMD_WAIT},
-{-1019, CMD_WAIT}, // numpad .
+{CK_NUMPAD_DECIMAL, CMD_WAIT},
 #ifdef USE_TILE_LOCAL
 {CK_DELETE, CMD_WAIT}, // numpad . on some keyboards on sdl??
 #endif
@@ -22,15 +22,15 @@
 {CK_PGDN, CMD_MOVE_DOWN_RIGHT},
 {CK_PGUP, CMD_MOVE_UP_RIGHT},
 {CK_HOME, CMD_MOVE_UP_LEFT},
-{-1001, CMD_MOVE_DOWN_LEFT},
-{-1004, CMD_MOVE_LEFT},
-{-1002, CMD_MOVE_DOWN},
-{-1008, CMD_MOVE_UP},
-{-1006, CMD_MOVE_RIGHT},
-{-1003, CMD_MOVE_DOWN_RIGHT},
-{-1009, CMD_MOVE_UP_RIGHT},
-{-1007, CMD_MOVE_UP_LEFT},
-{-1005, CMD_REST},
+{CK_NUMPAD_1, CMD_MOVE_DOWN_LEFT},
+{CK_NUMPAD_4, CMD_MOVE_LEFT},
+{CK_NUMPAD_2, CMD_MOVE_DOWN},
+{CK_NUMPAD_8, CMD_MOVE_UP},
+{CK_NUMPAD_6, CMD_MOVE_RIGHT},
+{CK_NUMPAD_3, CMD_MOVE_DOWN_RIGHT},
+{CK_NUMPAD_9, CMD_MOVE_UP_RIGHT},
+{CK_NUMPAD_7, CMD_MOVE_UP_LEFT},
+{CK_NUMPAD_5, CMD_REST},
 {'b', CMD_MOVE_DOWN_LEFT},
 {'h', CMD_MOVE_LEFT},
 {'j', CMD_MOVE_DOWN},
@@ -41,7 +41,9 @@
 {'y', CMD_MOVE_UP_LEFT},
 // other commands
 {'a', CMD_USE_ABILITY},
+{'c', CMD_UNEQUIP}, // not a very good letter for this
 {'d', CMD_DROP},
+{'e', CMD_EQUIP},
 {'f', CMD_FIRE},
 {'g', CMD_PICKUP},
 {'i', CMD_DISPLAY_INVENTORY},
@@ -77,7 +79,7 @@
 {'C', CMD_CLOSE_DOOR},
 {'D', CMD_DROP_LAST},
 {'E', CMD_EXPERIENCE_CHECK},
-{'F', CMD_THROW_ITEM_NO_QUIVER},
+{'F', CMD_FIRE_ITEM_NO_QUIVER},
 {CONTROL('G'), CMD_INTERLEVEL_TRAVEL},
 {'G', CMD_INTERLEVEL_TRAVEL},
 {'I', CMD_DISPLAY_SPELLS},
@@ -116,6 +118,10 @@
 {CONTROL('E'), CMD_MACRO_ADD},
 {'~', CMD_GAME_MENU},
 {CK_F1, CMD_GAME_MENU},
+#ifdef __ANDROID__
+{CK_F11, CMD_TOGGLE_TAB_ICONS},
+{CK_F12, CMD_TOGGLE_KEYBOARD},
+#endif
 #ifdef WIZARD
 {'&', CMD_WIZARD},
 {'+', CMD_EXPLORE_MODE},
@@ -193,9 +199,9 @@
 {':', CMD_TARGET_TOGGLE_BEAM},
 {'!', CMD_TARGET_SELECT_FORCE},
 {'@', CMD_TARGET_SELECT_FORCE_ENDPOINT},
-{'\r', CMD_TARGET_SELECT},
+{CK_ENTER, CMD_TARGET_SELECT},
 {'5', CMD_TARGET_SELECT},
-{-1019, CMD_TARGET_SELECT_ENDPOINT}, // numpad .
+{CK_NUMPAD_DECIMAL, CMD_TARGET_SELECT_ENDPOINT}, // numpad .
 {'.', CMD_TARGET_SELECT_ENDPOINT},
 {'e', CMD_TARGET_EXCLUDE},
 {'\t', CMD_TARGET_FIND_PORTAL},
@@ -209,17 +215,17 @@
 //{'f', CMD_TARGET_SELECT}, // HACK: Moved to be the downmost for tutorial key
                             // look-up.
 {'g', CMD_TARGET_GET},
-{-1018, CMD_TARGET_CYCLE_BACK}, // numpad -
-{-1020, CMD_TARGET_CYCLE_BACK}, // numpad -, maybe?
+{CK_NUMPAD_SUBTRACT, CMD_TARGET_CYCLE_BACK}, // numpad -
+{CK_NUMPAD_SUBTRACT2, CMD_TARGET_CYCLE_BACK}, // numpad -, maybe?
 {'-', CMD_TARGET_CYCLE_BACK},
 {'=', CMD_TARGET_CYCLE_FORWARD},
-{-1016, CMD_TARGET_CYCLE_FORWARD}, // numpad +
-{-1017, CMD_TARGET_CYCLE_FORWARD}, // numpad +, maybe?
+{CK_NUMPAD_ADD, CMD_TARGET_CYCLE_FORWARD}, // numpad +
+{CK_NUMPAD_ADD2, CMD_TARGET_CYCLE_FORWARD}, // numpad +, maybe?
 {'+', CMD_TARGET_CYCLE_FORWARD},
-{-1012, CMD_TARGET_OBJ_CYCLE_BACK}, // numpad /, sometimes
+{CK_NUMPAD_DIVIDE, CMD_TARGET_OBJ_CYCLE_BACK}, // numpad /, sometimes
 {'/', CMD_TARGET_OBJ_CYCLE_BACK},
 {';', CMD_TARGET_OBJ_CYCLE_BACK},
-{-1015, CMD_TARGET_OBJ_CYCLE_FORWARD}, // numpad *, sometimes
+{CK_NUMPAD_MULTIPLY, CMD_TARGET_OBJ_CYCLE_FORWARD}, // numpad *, sometimes
 {'*', CMD_TARGET_OBJ_CYCLE_FORWARD},
 {'\'', CMD_TARGET_OBJ_CYCLE_FORWARD},
 {'b', CMD_TARGET_DOWN_LEFT},
@@ -242,15 +248,16 @@
 {CK_PGDN, CMD_TARGET_DOWN_RIGHT},
 {CK_PGUP, CMD_TARGET_UP_RIGHT},
 {CK_HOME, CMD_TARGET_UP_LEFT},
-{-1001, CMD_TARGET_DOWN_LEFT},
-{-1004, CMD_TARGET_LEFT},
-{-1002, CMD_TARGET_DOWN},
-{-1008, CMD_TARGET_UP},
-{-1006, CMD_TARGET_RIGHT},
-{-1003, CMD_TARGET_DOWN_RIGHT},
-{-1009, CMD_TARGET_UP_RIGHT},
-{-1007, CMD_TARGET_UP_LEFT},
-{-1005, CMD_TARGET_SELECT},
+{CK_NUMPAD_1, CMD_TARGET_DOWN_LEFT},
+{CK_NUMPAD_4, CMD_TARGET_LEFT},
+{CK_NUMPAD_2, CMD_TARGET_DOWN},
+{CK_NUMPAD_8, CMD_TARGET_UP},
+{CK_NUMPAD_6, CMD_TARGET_RIGHT},
+{CK_NUMPAD_3, CMD_TARGET_DOWN_RIGHT},
+{CK_NUMPAD_9, CMD_TARGET_UP_RIGHT},
+{CK_NUMPAD_7, CMD_TARGET_UP_LEFT},
+{CK_NUMPAD_5, CMD_TARGET_SELECT},
+{CK_NUMPAD_ENTER, CMD_TARGET_SELECT},
 {'f', CMD_TARGET_SELECT}, // HACK: Down here for tutorial key look-up, see above
 {'B', CMD_TARGET_DIR_DOWN_LEFT},
 {'H', CMD_TARGET_DIR_LEFT},
@@ -279,6 +286,7 @@
 {CONTROL('F'), CMD_MAP_FORGET},
 {CONTROL('U'), CMD_MAP_UNFORGET},
 {CONTROL('W'), CMD_MAP_ADD_WAYPOINT},
+{'w', CMD_MAP_ADD_WAYPOINT},
 {'e', CMD_MAP_EXCLUDE_AREA},
 {CONTROL('E'), CMD_MAP_CLEAR_EXCLUDES},
 {'R', CMD_MAP_EXCLUDE_RADIUS},
@@ -298,14 +306,14 @@
 {CK_PGDN, CMD_MAP_MOVE_DOWN_RIGHT},
 {CK_PGUP, CMD_MAP_MOVE_UP_RIGHT},
 {CK_HOME, CMD_MAP_MOVE_UP_LEFT},
-{-1001, CMD_MAP_MOVE_DOWN_LEFT},
-{-1004, CMD_MAP_MOVE_LEFT},
-{-1002, CMD_MAP_MOVE_DOWN},
-{-1008, CMD_MAP_MOVE_UP},
-{-1006, CMD_MAP_MOVE_RIGHT},
-{-1003, CMD_MAP_MOVE_DOWN_RIGHT},
-{-1009, CMD_MAP_MOVE_UP_RIGHT},
-{-1007, CMD_MAP_MOVE_UP_LEFT},
+{CK_NUMPAD_1, CMD_MAP_MOVE_DOWN_LEFT},
+{CK_NUMPAD_4, CMD_MAP_MOVE_LEFT},
+{CK_NUMPAD_2, CMD_MAP_MOVE_DOWN},
+{CK_NUMPAD_8, CMD_MAP_MOVE_UP},
+{CK_NUMPAD_6, CMD_MAP_MOVE_RIGHT},
+{CK_NUMPAD_3, CMD_MAP_MOVE_DOWN_RIGHT},
+{CK_NUMPAD_9, CMD_MAP_MOVE_UP_RIGHT},
+{CK_NUMPAD_7, CMD_MAP_MOVE_UP_LEFT},
 {'B', CMD_MAP_JUMP_DOWN_LEFT},
 {'H', CMD_MAP_JUMP_LEFT},
 {'J', CMD_MAP_JUMP_DOWN},
@@ -325,11 +333,12 @@
 {'[', CMD_MAP_PREV_LEVEL},
 {']', CMD_MAP_NEXT_LEVEL},
 {'G', CMD_MAP_GOTO_LEVEL},
-{-1016, CMD_MAP_SCROLL_DOWN}, // numpad +
-{-1017, CMD_MAP_SCROLL_DOWN}, // numpad +, maybe
+{CK_NUMPAD_ADD, CMD_MAP_SCROLL_DOWN}, // numpad +
+{CK_NUMPAD_ADD2, CMD_MAP_SCROLL_DOWN}, // numpad +, maybe
 {'+', CMD_MAP_SCROLL_DOWN},
 {CK_MOUSE_B5, CMD_MAP_SCROLL_DOWN},
-{-1018, CMD_MAP_SCROLL_UP},
+{CK_NUMPAD_SUBTRACT, CMD_MAP_SCROLL_UP},
+{CK_NUMPAD_SUBTRACT2, CMD_MAP_SCROLL_UP},
 {'-', CMD_MAP_SCROLL_UP},
 {CK_MOUSE_B4, CMD_MAP_SCROLL_UP},
 {'<', CMD_MAP_FIND_UPSTAIR},
@@ -345,9 +354,10 @@
 {'I', CMD_MAP_FIND_STASH},
 {'O', CMD_MAP_FIND_STASH_REVERSE},
 {'\'', CMD_MAP_FIND_STASH},
-{-1019, CMD_MAP_GOTO_TARGET}, // numpad .
+{CK_NUMPAD_DECIMAL, CMD_MAP_GOTO_TARGET}, // numpad .
 {'.', CMD_MAP_GOTO_TARGET},
-{'\r', CMD_MAP_GOTO_TARGET},
+{CK_NUMPAD_ENTER, CMD_MAP_GOTO_TARGET},
+{CK_ENTER, CMD_MAP_GOTO_TARGET},
 {'S', CMD_MAP_GOTO_TARGET},
 {',', CMD_MAP_GOTO_TARGET},
 {';', CMD_MAP_GOTO_TARGET},
@@ -355,7 +365,7 @@
 {'o', CMD_MAP_EXPLORE},
 {'v', CMD_MAP_DESCRIBE},
 {ESCAPE, CMD_MAP_EXIT_MAP},
-{CONTROL('I'), CMD_AUTOFIGHT},
+{CONTROL('I'), CMD_AUTOFIGHT}, // tab
 {CK_SHIFT_TAB, CMD_AUTOFIRE}, // also 'p'
 
 #ifdef WIZARD
@@ -363,6 +373,45 @@
 {CONTROL('X'), CMD_MAP_WIZARD_FORGET},
 #endif
 
+// N.b. these cannot be rebound on webtiles
+// numpad mappings are handled (by default) in Menu via numpad_to_regular
+{CK_UP, CMD_MENU_UP},
+{CK_DOWN, CMD_MENU_DOWN},
+{CK_SHIFT_UP, CMD_MENU_LINE_UP},
+{CK_RIGHT, CMD_MENU_RIGHT},
+{CK_LEFT, CMD_MENU_LEFT},
+{CK_SHIFT_DOWN, CMD_MENU_LINE_DOWN},
+{CK_SHIFT_LEFT, CMD_MENU_LINE_UP}, // some terms don't have shift-up/down available by default
+{CK_SHIFT_RIGHT, CMD_MENU_LINE_DOWN},
+{'<', CMD_MENU_PAGE_UP},
+{'-', CMD_MENU_PAGE_UP}, // manually overridden by many menus
+{CK_PGUP, CMD_MENU_PAGE_UP},
+{' ', CMD_MENU_PAGE_DOWN}, // currently still can be overridden with hacks
+{'>', CMD_MENU_PAGE_DOWN},
+{'+', CMD_MENU_PAGE_DOWN},
+{CK_PGDN, CMD_MENU_PAGE_DOWN},
+{CK_HOME, CMD_MENU_SCROLL_TO_TOP},
+{CK_END, CMD_MENU_SCROLL_TO_END},
+{CONTROL('F'), CMD_MENU_SEARCH},
+{'!', CMD_MENU_CYCLE_MODE},
+{',', CMD_MENU_CYCLE_HEADERS},
+{'_', CMD_MENU_HELP},
+{'?', CMD_MENU_HELP}, // overridden in item menus
+{CK_ENTER, CMD_MENU_SELECT},
+{'\\', CMD_MENU_EXAMINE},
+{'\'', CMD_MENU_EXAMINE},
+{CK_MOUSE_B2, CMD_MENU_EXIT},
+{CK_MOUSE_CMD, CMD_MENU_EXIT},
+{CONTROL('G'), CMD_MENU_EXIT},
+{ESCAPE, CMD_MENU_EXIT},
+
+// multiselect-specific:
+{CK_ENTER, CMD_MENU_ACCEPT_SELECTION},
+{',', CMD_MENU_SELECT_ALL},
+{'*', CMD_MENU_INVERT_SELECTION},
+{'-', CMD_MENU_CLEAR_SELECTION},
+{'.', CMD_MENU_TOGGLE_SELECTED}, // TODO: ' '
+
 #ifdef USE_TILE
 {CONTROL('R'), CMD_DOLL_RANDOMIZE},
 {'H', CMD_DOLL_SELECT_PREV_DOLL},
@@ -377,10 +426,11 @@
 {CK_DOWN, CMD_DOLL_SELECT_NEXT_PART},
 {CK_LEFT, CMD_DOLL_CHANGE_PART_PREV},
 {CK_RIGHT, CMD_DOLL_CHANGE_PART_NEXT},
-{-1008, CMD_DOLL_SELECT_PREV_PART},
-{-1002, CMD_DOLL_SELECT_NEXT_PART},
-{-1004, CMD_DOLL_CHANGE_PART_PREV},
-{-1006, CMD_DOLL_CHANGE_PART_NEXT},
+{CK_NUMPAD_8, CMD_DOLL_SELECT_PREV_PART},
+{CK_NUMPAD_2, CMD_DOLL_SELECT_NEXT_PART},
+{CK_NUMPAD_4, CMD_DOLL_CHANGE_PART_PREV},
+{CK_NUMPAD_6, CMD_DOLL_CHANGE_PART_NEXT},
+{CK_NUMPAD_ENTER, CMD_DOLL_CONFIRM_CHOICE},
 {CK_ENTER, CMD_DOLL_CONFIRM_CHOICE},
 {CONTROL('C'), CMD_DOLL_COPY},
 {CONTROL('V'), CMD_DOLL_PASTE},
@@ -396,20 +446,17 @@
 {CONTROL('Q'), CMD_DOLL_QUIT},
 #endif
 
-#ifdef TOUCH_UI
-// zoom functions
-{CK_NUMPAD_PLUS, CMD_ZOOM_IN},
-{CK_NUMPAD_MINUS, CMD_ZOOM_OUT},
-#elif defined(USE_TILE_LOCAL)
+#if defined(USE_TILE_LOCAL)
 // no good webtiles keys available for the main view case, and browser zoom
 // already more or less accomplishes this.
-{'=' - SDLK_a + 1, CMD_ZOOM_IN},  // Believe it or not, this is how we map SDL
-{'+' - SDLK_a + 1, CMD_ZOOM_IN},  // ctrl scancodes...
-{'-' - SDLK_a + 1, CMD_ZOOM_OUT},
-{'=' - SDLK_a + 1, CMD_MAP_ZOOM_IN},
-{'+' - SDLK_a + 1, CMD_MAP_ZOOM_IN},
-{'-' - SDLK_a + 1, CMD_MAP_ZOOM_OUT},
+{LC_CONTROL('='), CMD_ZOOM_IN},
+{LC_CONTROL('+'), CMD_ZOOM_IN},
+{LC_CONTROL('-'), CMD_ZOOM_OUT},
+{LC_CONTROL('='), CMD_MAP_ZOOM_IN},
+{LC_CONTROL('+'), CMD_MAP_ZOOM_IN},
+{LC_CONTROL('-'), CMD_MAP_ZOOM_OUT},
 #endif
+
 #ifdef USE_TILE
 {'}', CMD_MAP_ZOOM_IN},
 {'{', CMD_MAP_ZOOM_OUT},
