@@ -294,7 +294,9 @@ static int crawl_yesno(lua_State *ls)
 
 static void crawl_sendkeys_proc(lua_State *ls, int argi)
 {
-    if (lua_isstring(ls, argi))
+    if (lua_type(ls, argi) == LUA_TNUMBER)
+        macro_sendkeys_end_add_expanded(luaL_safe_checkint(ls, argi));
+    else if (lua_isstring(ls, argi))
     {
         const char *keys = luaL_checkstring(ls, argi);
         if (!keys)
@@ -322,8 +324,6 @@ static void crawl_sendkeys_proc(lua_State *ls, int argi)
             lua_pop(ls, 1);
         }
     }
-    else if (lua_isnumber(ls, argi))
-        macro_sendkeys_end_add_expanded(luaL_safe_checkint(ls, argi));
 }
 
 /*** Send keypresses to crawl.
@@ -423,10 +423,10 @@ static int crawl_do_targeted_command(lua_State *ls)
     case CMD_FIRE:
         quiver::get_secondary_action()->trigger(target);
         break;
-    case CMD_THROW_ITEM_NO_QUIVER:
+    case CMD_FIRE_ITEM_NO_QUIVER:
         // This pops up an inventory menu -- maybe support taking an item
         // directly?
-        throw_item_no_quiver(&target);
+        fire_item_no_quiver(&target);
         break;
     default:
         luaL_argerror(ls, 1, ("Not a (supported) targeted command: " + command).c_str());
@@ -1248,21 +1248,6 @@ static int crawl_is_webtiles(lua_State *ls)
     return 1;
 }
 
-/*** Are we using the touch ui?
- * @treturn boolean
- * @function is_touch_ui
- */
-static int crawl_is_touch_ui(lua_State *ls)
-{
-#ifdef TOUCH_UI
-    lua_pushboolean(ls, true);
-#else
-    lua_pushboolean(ls, false);
-#endif
-
-    return 1;
-}
-
 /*** Look up the current key bound to a command.
  * @tparam string name Name as in cmd-name.h
  * @treturn string|nil
@@ -1286,7 +1271,6 @@ static int crawl_get_command(lua_State *ls)
     return 1;
 }
 
-LUAWRAP(crawl_endgame, screen_end_game(luaL_checkstring(ls, 1)))
 LUAWRAP(crawl_tutorial_skill, set_tutorial_skill(luaL_checkstring(ls, 1), luaL_safe_checkint(ls, 2)))
 LUAWRAP(crawl_tutorial_hint, tutorial_init_hint(luaL_checkstring(ls, 1)))
 LUAWRAP(crawl_print_hint, print_hint(luaL_checkstring(ls, 1), luaL_optstring(ls, 2, ""), luaL_optstring(ls, 3, "")))
@@ -1435,6 +1419,30 @@ static int crawl_version(lua_State *ls)
     return 1;
 }
 
+LUAFN(crawl_hints_type)
+{
+    if (crawl_state.game_is_tutorial())
+        lua_pushstring(ls, "tutorial");
+    else if (!crawl_state.game_is_hints())
+        lua_pushstring(ls, "");
+    else
+        switch (Hints.hints_type)
+        {
+        case HINT_BERSERK_CHAR:
+            lua_pushstring(ls, "berserk");
+            break;
+        case HINT_RANGER_CHAR:
+            lua_pushstring(ls, "ranger");
+            break;
+        case HINT_MAGIC_CHAR:
+            lua_pushstring(ls, "magic");
+            break;
+        default:
+            die("invalid hints_type");
+        }
+    return 1;
+}
+
 static const struct luaL_reg crawl_clib[] =
 {
     { "mpr",                crawl_mpr },
@@ -1496,10 +1504,8 @@ static const struct luaL_reg crawl_clib[] =
     { "stat_gain_prompt",   crawl_stat_gain_prompt },
     { "is_tiles",           crawl_is_tiles },
     { "is_webtiles",        crawl_is_webtiles },
-    { "is_touch_ui",        crawl_is_touch_ui },
     { "err_trace",          crawl_err_trace },
     { "get_command",        crawl_get_command },
-    { "endgame",            crawl_endgame },
     { "tutorial_msg",       crawl_tutorial_msg },
     { "dump_char",          crawl_dump_char },
 #ifdef WIZARD
@@ -1507,6 +1513,7 @@ static const struct luaL_reg crawl_clib[] =
 #endif
     { "version",            crawl_version },
     { "weapon_check",       crawl_weapon_check},
+    { "hints_type",         crawl_hints_type },
     { nullptr, nullptr },
 };
 
@@ -1717,30 +1724,6 @@ LUAFN(_crawl_set_max_runes)
 
 LUAWRAP(_crawl_mark_game_won, crawl_state.mark_last_game_won())
 
-LUAFN(crawl_hints_type)
-{
-    if (crawl_state.game_is_tutorial())
-        lua_pushstring(ls, "tutorial");
-    else if (!crawl_state.game_is_hints())
-        lua_pushstring(ls, "");
-    else
-        switch (Hints.hints_type)
-        {
-        case HINT_BERSERK_CHAR:
-            lua_pushstring(ls, "berserk");
-            break;
-        case HINT_RANGER_CHAR:
-            lua_pushstring(ls, "ranger");
-            break;
-        case HINT_MAGIC_CHAR:
-            lua_pushstring(ls, "magic");
-            break;
-        default:
-            die("invalid hints_type");
-        }
-    return 1;
-}
-
 LUAFN(crawl_rng_wrap)
 {
     if (!lua_isstring(ls, 2))
@@ -1787,6 +1770,11 @@ LUAFN(crawl_rng_wrap)
 
 LUAWRAP(crawl_clear_message_store, clear_message_store())
 
+/*** Whether the crawl process has seen a HUP or INT signal.
+ * @treturn int the number of hups seen
+ * @function seen_hups
+ */
+LUARET1(crawl_seen_hups, number, crawl_state.seen_hups)
 
 static const struct luaL_reg crawl_dlib[] =
 {
@@ -1802,10 +1790,10 @@ static const struct luaL_reg crawl_dlib[] =
 { "tutorial_hint",   crawl_tutorial_hint },
 { "print_hint", crawl_print_hint },
 { "mark_game_won", _crawl_mark_game_won },
-{ "hints_type", crawl_hints_type },
 { "unavailable_god", _crawl_unavailable_god },
 { "rng_wrap", crawl_rng_wrap },
 { "clear_message_store", crawl_clear_message_store },
+{ "seen_hups", crawl_seen_hups },
 
 { nullptr, nullptr }
 };
