@@ -173,6 +173,8 @@ struct mgen_data
         ASSERT(summon_type == 0 || abjuration_dur >= 1 && abjuration_dur <= 6
                || cls == MONS_BALL_LIGHTNING || cls == MONS_ORB_OF_DESTRUCTION
                || cls == MONS_BATTLESPHERE || cls == MONS_BALLISTOMYCETE_SPORE
+               || summon_type == SPELL_ANIMATE_DEAD
+               || summon_type == SPELL_NECROTISE
                || summon_type == SPELL_DEATH_CHANNEL
                || summon_type == SPELL_BIND_SOULS
                || summon_type == SPELL_SIMULACRUM
