@@ -79,7 +79,7 @@ void maybe_melt_player_enchantments(beam_type flavour, int damage)
             if (!you.duration[DUR_ICEMAIL_DEPLETED])
             {
                 if (you.has_mutation(MUT_ICEMAIL))
-                    mprf(MSGCH_DURATION, "Your icy defenses dissipate!");
+                    mprf(MSGCH_DURATION, "Your icy defences dissipate!");
                 else
                     mprf(MSGCH_DURATION, "Your condensation shield dissipates!");
             }
@@ -345,10 +345,9 @@ void lose_level()
     calc_hp();
     calc_mp();
 
-    char buf[200];
-    sprintf(buf, "HP: %d/%d MP: %d/%d",
-            you.hp, you.hp_max, you.magic_points, you.max_magic_points);
-    take_note(Note(NOTE_XP_LEVEL_CHANGE, you.experience_level, 0, buf));
+    take_note(Note(NOTE_XP_LEVEL_CHANGE, you.experience_level, 0,
+        make_stringf("HP: %d/%d MP: %d/%d",
+                you.hp, you.hp_max, you.magic_points, you.max_magic_points)));
 
     you.redraw_title = true;
     you.redraw_experience = true;
@@ -580,7 +579,7 @@ static void _maybe_spawn_monsters(int dam, kill_method_type death_type,
     monster* damager = monster_by_mid(death_source);
     // We need to exclude acid damage and similar things or this function
     // will crash later.
-    if (!damager)
+    if (!damager || death_source == MID_YOU_FAULTLESS)
         return;
 
     monster_type mon;
@@ -609,7 +608,8 @@ static void _maybe_spawn_monsters(int dam, kill_method_type death_type,
         int count_created = 0;
         for (int i = 0; i < how_many; ++i)
         {
-            mgen_data mg(mon, BEH_FRIENDLY, you.pos(), damager->mindex());
+            const int mindex = damager->alive() ? damager->mindex() : MHITNOT;
+            mgen_data mg(mon, BEH_FRIENDLY, you.pos(), mindex);
             mg.set_summoned(&you, 2, 0, you.religion);
 
             if (create_monster(mg))
@@ -713,15 +713,19 @@ static void _deteriorate(int dam)
     }
 }
 
+int corrosion_chance(int sources)
+{
+    return 3 * sources;
+}
+
 /**
  * Maybe corrode the player after taking damage if they're wearing *Corrode.
  **/
 static void _maybe_corrode()
 {
     int corrosion_sources = you.scan_artefacts(ARTP_CORRODE);
-    int degree = binomial(corrosion_sources, 3);
-    if (degree > 0)
-        you.corrode_equipment("Your corrosive artefact", degree);
+    if (x_chance_in_y(corrosion_chance(corrosion_sources), 100))
+        you.corrode_equipment("Your corrosive artefact");
 }
 
 /**
@@ -730,7 +734,7 @@ static void _maybe_corrode()
 static void _maybe_slow()
 {
     int slow_sources = you.scan_artefacts(ARTP_SLOW);
-    for (int degree = binomial(slow_sources, 1); degree > 0; degree--)
+    if (x_chance_in_y(slow_sources, 100))
         slow_player(10 + random2(5));
 }
 
@@ -767,14 +771,25 @@ static void _wizard_restore_life()
 }
 #endif
 
+int outgoing_harm_amount(int levels)
+{
+    return 15 * (levels + 1);
+}
+
+int incoming_harm_amount(int levels)
+{
+    return 10 * (levels + 1);
+}
+
 static int _apply_extra_harm(int dam, mid_t source)
 {
     monster* damager = monster_by_mid(source);
-    // Don't check for monster amulet if there source isn't a monster
+    // +30% damage if opp has one level of harm, +45% with two
     if (damager && damager->extra_harm())
-        return dam * 13 / 10; // +30% damage when the opponent has harm
-    else if (you.extra_harm())
-        return dam * 6 / 5; // +20% damage when you have harm
+        return dam * (100 + outgoing_harm_amount(damager->extra_harm())) / 100;
+    // +20% damage if you have one level of harm, +30% with two
+    if (you.extra_harm())
+        return dam * (100 + incoming_harm_amount(you.extra_harm())) / 100;
 
     return dam;
 }
@@ -786,9 +801,10 @@ void reset_damage_counters()
     you.source_damage = 0;
 }
 
+#if TAG_MAJOR_VERSION == 34
 bool can_shave_damage()
 {
-    return you.species == SP_DEEP_DWARF;
+    return you.species == SP_DEEP_DWARF || you.species == SP_MAYFLYTAUR;
 }
 
 int do_shave_damage(int dam)
@@ -803,52 +819,24 @@ int do_shave_damage(int dam)
 
     return dam;
 }
+#endif
 
-// Determine what's threatening for purposes of sacrifice drink and reading.
-// the statuses are guaranteed not to happen if the incoming damage is less
-// than 5% max hp. Otherwise, they scale up with damage taken and with lower
-// health, becoming certain at 20% max health damage.
-static bool _is_damage_threatening (int damage_fraction_of_hp)
+// Determine what's threatening for purposes of no drink and no scroll mutation.
+// The statuses are guaranteed not to happen if the incoming damage is less
+// than 12/5% max hp and if the remaining hp is higher than 50/80% based on the
+// mutation tier. Otherwise, they scale up with damage taken and with lower
+// health, becoming certain at 50/20% max health damage.
+static bool _is_damage_threatening (int damage_fraction_of_hp, int mut_level)
 {
     const int hp_fraction = you.hp * 100 / you.hp_max;
-    return damage_fraction_of_hp > 5
-            && hp_fraction <= 85
-            && (damage_fraction_of_hp + random2(20) >= 20
+    const int safe_damage_fraction = mut_level == 1 ? 12 : 5;
+    const int scary_damage_fraction = mut_level == 1 ? 50 : 20;
+    return damage_fraction_of_hp > safe_damage_fraction
+            && hp_fraction <= 100 - scary_damage_fraction + safe_damage_fraction
+            && (damage_fraction_of_hp + random2(scary_damage_fraction) >= scary_damage_fraction
                 || random2(100) > hp_fraction);
 }
 
-// Palentongas curl up after the first time they've been hit in a round.
-static void _consider_curling(kill_method_type death_type)
-{
-    if (!you.has_mutation(MUT_CURL)
-        || you.props[PALENTONGA_CURL_KEY].get_bool())
-    {
-        return;
-    }
-
-    switch (death_type)
-    {
-        case KILLED_BY_MONSTER:
-        case KILLED_BY_BEAM:
-        case KILLED_BY_DEATH_EXPLOSION:
-        case KILLED_BY_TRAP:
-        case KILLED_BY_BOUNCE:
-        case KILLED_BY_REFLECTION:
-        case KILLED_BY_DISINT:
-        case KILLED_BY_HEADBUTT:
-        case KILLED_BY_ROLLING:
-        case KILLED_BY_BEING_THROWN:
-        case KILLED_BY_COLLISION:
-            break;
-        default:
-            // stuff like poison, smiting, etc
-            return;
-    }
-
-    you.props[PALENTONGA_CURL_KEY] = true;
-    you.redraw_armour_class = true;
-}
-
 /** Hurt the player. Isn't it fun?
  *
  *  @param dam How much damage -- may be INSTANT_DEATH.
@@ -870,15 +858,17 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
 
     int drain_amount = 0;
 
-    // Multiply damage if scarf of harm is in play
+    // Multiply damage if Harm is in play
     if (dam != INSTANT_DEATH)
         dam = _apply_extra_harm(dam, source);
 
+#if TAG_MAJOR_VERSION == 34
     if (can_shave_damage() && dam != INSTANT_DEATH
         && death_type != KILLED_BY_POISON)
     {
         dam = max(0, do_shave_damage(dam));
     }
+#endif
 
     if (dam != INSTANT_DEATH)
     {
@@ -901,8 +891,6 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
     if (dam > 0 && dam < you.hp && death_type != KILLED_BY_POISON)
         you.check_awaken(500);
 
-    _consider_curling(death_type);
-
     const bool non_death = death_type == KILLED_BY_QUITTING
                         || death_type == KILLED_BY_WINNING
                         || death_type == KILLED_BY_LEAVING;
@@ -942,7 +930,8 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
         // don't always trigger in unison when you have both.
         if (you.get_mutation_level(MUT_READ_SAFETY))
         {
-            if (_is_damage_threatening(damage_fraction_of_hp))
+            if (_is_damage_threatening(damage_fraction_of_hp,
+                                       you.get_mutation_level(MUT_READ_SAFETY)))
             {
                 if (!you.duration[DUR_NO_SCROLLS])
                     mpr("You feel threatened and lose the ability to read scrolls!");
@@ -953,7 +942,8 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
 
         if (you.get_mutation_level(MUT_DRINK_SAFETY))
         {
-            if (_is_damage_threatening(damage_fraction_of_hp))
+            if (_is_damage_threatening(damage_fraction_of_hp,
+                                       you.get_mutation_level(MUT_DRINK_SAFETY)))
             {
                 if (!you.duration[DUR_NO_POTIONS])
                     mpr("You feel threatened and lose the ability to drink potions!");
@@ -1054,7 +1044,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
                 drain_player(drain_amount, true, true);
         }
         if (you.hp > 0)
-          return;
+            return;
     }
 
     // Is the player being killed by a direct act of Xom?
@@ -1139,7 +1129,8 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
         if (!non_death)
             tutorial_death_message();
 
-        screen_end_game("");
+        // only quitting can lead to this?
+        screen_end_game("", game_exit::quit);
     }
 
     // Okay, so you're dead.
@@ -1165,7 +1156,8 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
         xom_death_message((kill_method_type) se.get_death_type());
         more();
 
-        _place_player_corpse(death_type == KILLED_BY_DISINT);
+        if (death_type != KILLED_BY_ZOT)
+            _place_player_corpse(death_type == KILLED_BY_DISINT);
         return;
     }
 
