@@ -175,7 +175,9 @@ static void _update_feat_at(const coord_def &gp)
     // so we can properly recolor both.
     if (env.level_state & LSTATE_ICY_WALL
         && (is_icecovered(gp)
-            || !feat_is_wall(feat) && count_adjacent_icy_walls(gp)))
+            || !feat_is_wall(feat)
+                && count_adjacent_icy_walls(gp)
+                && you.see_cell_no_trans(gp)))
     {
         env.map_knowledge(gp).flags |= MAP_ICY;
     }
@@ -540,7 +542,7 @@ void show_init(layers_type layers)
     ASSERT(you.on_current_level);
 
     vector <coord_def> update_locs;
-    for (radius_iterator ri(you.pos(), you.xray_vision ? LOS_NONE : LOS_DEFAULT); ri; ++ri)
+    for (vision_iterator ri(you); ri; ++ri)
     {
         show_update_at(*ri, layers);
         update_locs.push_back(*ri);
