@@ -88,7 +88,7 @@ bool save_doll_data(int mode, int num, const dolls_data* dolls)
         fprintf(fp, "#Legend:\n");
         fprintf(fp, "#***:equipment/123:index/000:none\n");
         fprintf(fp, "#Shadow/Base/Cloak/Boots/Legs/Body/Gloves/Weapon/Shield/"
-                    "Hair/Beard/Helmet/Halo/Enchant/DrcHead/DrcWing\n");
+                    "Hair/Beard/Helmet/Halo/Enchant/DrcWing\n");
         fprintf(fp, "#Sh:Bse:Clk:Bts:Leg:Bdy:Glv:Wpn:Shd:Hai:Brd:Hlm:Hal:Enc:Drc:Wng\n");
         char fbuf[80];
         for (unsigned int i = 0; i < NUM_MAX_DOLLS; ++i)
@@ -281,7 +281,6 @@ void fill_doll_equipment(dolls_data &result)
     case transformation::tree:
         result.parts[TILEP_PART_BASE]    = TILEP_TRAN_TREE;
         result.parts[TILEP_PART_HELM]    = 0; // fixme, should show up
-        result.parts[TILEP_PART_DRCHEAD] = 0;
         result.parts[TILEP_PART_DRCWING] = 0;
         result.parts[TILEP_PART_HAIR]    = 0;
         result.parts[TILEP_PART_BEARD]   = 0;
@@ -295,7 +294,7 @@ void fill_doll_equipment(dolls_data &result)
 #if TAG_MAJOR_VERSION == 34
         case SP_CENTAUR:
 #endif
-        case SP_PALENTONGA: ch = TILEP_TRAN_STATUE_PALENTONGA;  break;
+        case SP_ARMATAUR: ch = TILEP_TRAN_STATUE_ARMATAUR;  break;
         case SP_NAGA:    ch = TILEP_TRAN_STATUE_NAGA;     break;
         case SP_FELID:   ch = TILEP_TRAN_STATUE_FELID;    break;
         case SP_OCTOPODE:ch = TILEP_TRAN_STATUE_OCTOPODE; break;
@@ -303,7 +302,6 @@ void fill_doll_equipment(dolls_data &result)
         default:         ch = TILEP_TRAN_STATUE_HUMANOID; break;
         }
         result.parts[TILEP_PART_BASE]    = ch;
-        result.parts[TILEP_PART_DRCHEAD] = 0;
         result.parts[TILEP_PART_HAIR]    = 0;
         result.parts[TILEP_PART_LEG]     = 0;
         break;
@@ -316,7 +314,6 @@ void fill_doll_equipment(dolls_data &result)
         }
         result.parts[TILEP_PART_BASE]    = ch;
         result.parts[TILEP_PART_HELM]    = 0;
-        result.parts[TILEP_PART_DRCHEAD] = 0;
         result.parts[TILEP_PART_DRCWING] = 0;
         result.parts[TILEP_PART_HAIR]    = 0;
         result.parts[TILEP_PART_BEARD]   = 0;
@@ -333,7 +330,7 @@ void fill_doll_equipment(dolls_data &result)
 #if TAG_MAJOR_VERSION == 34
         case SP_CENTAUR:
 #endif
-        case SP_PALENTONGA: ch = TILEP_TRAN_LICH_PALENTONGA;  break;
+        case SP_ARMATAUR: ch = TILEP_TRAN_LICH_ARMATAUR;  break;
         case SP_NAGA:    ch = TILEP_TRAN_LICH_NAGA;     break;
         case SP_FELID:   ch = TILEP_TRAN_LICH_FELID;    break;
         case SP_OCTOPODE:ch = TILEP_TRAN_LICH_OCTOPODE; break;
@@ -341,7 +338,6 @@ void fill_doll_equipment(dolls_data &result)
         default:         ch = TILEP_TRAN_LICH_HUMANOID; break;
         }
         result.parts[TILEP_PART_BASE]    = ch;
-        result.parts[TILEP_PART_DRCHEAD] = 0;
         result.parts[TILEP_PART_HAIR]    = 0;
         result.parts[TILEP_PART_BEARD]   = 0;
         result.parts[TILEP_PART_LEG]     = 0;
@@ -358,7 +354,6 @@ void fill_doll_equipment(dolls_data &result)
         if (Options.tile_use_monster != MONS_0)
         {
             result.parts[TILEP_PART_BASE]    = tileidx_player_mons();
-            result.parts[TILEP_PART_DRCHEAD] = 0;
             result.parts[TILEP_PART_HAIR]    = 0;
             result.parts[TILEP_PART_BEARD]   = 0;
             result.parts[TILEP_PART_LEG]     = 0;
@@ -449,7 +444,7 @@ void fill_doll_equipment(dolls_data &result)
                 else if (is_player_tile(result.parts[TILEP_PART_BASE],
                                   TILEP_TRAN_STATUE_FELID))
                 {
-                    result.parts[TILEP_PART_HELM] = TILEP_HELM_HORNS_CAT;
+                    result.parts[TILEP_PART_HELM] = TILEP_HELM_HORNS_CAT_STATUE;
                 }
             }
             else if (species::is_draconian(you.species))
@@ -515,13 +510,9 @@ void fill_doll_equipment(dolls_data &result)
     if (species::is_draconian(you.species))
     {
         tileidx_t base = 0;
-        tileidx_t head = 0;
         tileidx_t wing = 0;
-        tilep_draconian_init(you.species, you.experience_level,
-                             &base, &head, &wing);
+        tilep_draconian_init(you.species, you.experience_level, &base, &wing);
 
-        if (result.parts[TILEP_PART_DRCHEAD] == TILEP_SHOW_EQUIP)
-            result.parts[TILEP_PART_DRCHEAD] = head;
         if (result.parts[TILEP_PART_DRCWING] == TILEP_SHOW_EQUIP)
             result.parts[TILEP_PART_DRCWING] = wing;
     }
@@ -598,7 +589,6 @@ void pack_doll_buf(SubmergedTileBuffer& buf, const dolls_data &doll,
         TILEP_PART_ARM,
         TILEP_PART_HAIR,
         TILEP_PART_BEARD,
-        TILEP_PART_DRCHEAD,
         TILEP_PART_HELM,
         TILEP_PART_HAND1,
         TILEP_PART_HAND2
@@ -637,7 +627,7 @@ void pack_doll_buf(SubmergedTileBuffer& buf, const dolls_data &doll,
     }
 
     const bool is_ptng = is_player_tile(doll.parts[TILEP_PART_BASE],
-                                        TILEP_BASE_PALENTONGA);
+                                        TILEP_BASE_ARMATAUR);
 
     if (doll.parts[TILEP_PART_BOOTS] >= TILEP_BOOTS_CENTAUR_BARDING
         && doll.parts[TILEP_PART_BOOTS] <= TILEP_BOOTS_CENTAUR_BARDING_RED
