@@ -63,10 +63,6 @@ int AbilityRegion::handle_mouse(wm_mouse_event &event)
     const ability_type ability = (ability_type) m_items[item_idx].idx;
     if (event.button == wm_mouse_event::LEFT)
     {
-        // close tab again if using small layout
-        if (tiles.is_using_small_layout())
-            tiles.deactivate_tab();
-
         m_last_clicked_item = item_idx;
         tiles.set_need_redraw();
         // TODO get_talent returns ABIL_NON_ABILITY if you are confused,
@@ -152,9 +148,10 @@ bool AbilityRegion::update_alt_text(string &alt)
 
 int AbilityRegion::get_max_slots()
 {
-    return ABIL_MAX_INTRINSIC + (ABIL_MAX_EVOKE - ABIL_MIN_EVOKE) + 1
-           // for god abilities
-           + 6;
+    const int MAX_INTRINSICS = 3;
+    const int MAX_GOD_ABILS = 6;
+    const int MAX_EVOKES = 6; // TODO: don't hardcode this
+    return MAX_INTRINSICS + MAX_GOD_ABILS + MAX_EVOKES;
 }
 
 void AbilityRegion::pack_buffers()
