@@ -78,9 +78,9 @@ string Form::melding_description() const
     else if ((blocked_slots & EQF_PHYSICAL) == EQF_PHYSICAL)
         return "Your equipment is almost entirely melded.";
     else if ((blocked_slots & EQF_STATUE) == EQF_STATUE
-             && (you_can_wear(EQ_GLOVES, false)
-                 || you_can_wear(EQ_BOOTS, false)
-                 || you_can_wear(EQ_BODY_ARMOUR, false)))
+             && (you_can_wear(EQ_GLOVES, false) != false
+                 || you_can_wear(EQ_BOOTS, false) != false
+                 || you_can_wear(EQ_BODY_ARMOUR, false) != false))
     {
         return "Your equipment is partially melded.";
     }
@@ -590,12 +590,13 @@ public:
      */
     string transform_message(transformation previous_trans) const override
     {
+#if TAG_MAJOR_VERSION == 34
         if (you.species == SP_DEEP_DWARF && one_chance_in(10))
             return "You inwardly fear your resemblance to a lawn ornament.";
-        else if (you.species == SP_GARGOYLE)
+#endif
+        if (you.species == SP_GARGOYLE)
             return "Your body stiffens and grows slower.";
-        else
-            return Form::transform_message(previous_trans);
+        return Form::transform_message(previous_trans);
     }
 
     /**
@@ -1462,6 +1463,10 @@ static bool _flying_in_new_form(transformation which_trans)
     if (!you.duration[DUR_FLIGHT] && !you.attribute[ATTR_PERM_FLIGHT])
         return false;
 
+    // tempflight (e.g. from potion) enabled, no need for equip check
+    if (you.duration[DUR_FLIGHT])
+        return true;
+
     // Finally, do the calculation about what would be melded: are there equip
     // sources left?
     int sources = you.equip_flight();
@@ -1534,7 +1539,7 @@ static bool _transformation_is_safe(transformation which_trans,
                                     dungeon_feature_type feat,
                                     string *fail_reason)
 {
-    if (!feat_dangerous_for_form(which_trans, feat) || you.duration[DUR_FLIGHT])
+    if (!feat_dangerous_for_form(which_trans, feat))
         return true;
 
     if (fail_reason)
@@ -1857,25 +1862,31 @@ bool transform(int pow, transformation which_trans, bool involuntary,
             mpr("You feel strangely stable.");
         }
         you.duration[DUR_FLIGHT] = 0;
-        // break out of webs/nets as well
+
+        if (you.attribute[ATTR_HELD])
+        {
+            const trap_def *trap = trap_at(you.pos());
+            if (trap && trap->type == TRAP_WEB)
+            {
+                leave_web(true);
+                if (trap_at(you.pos()))
+                    mpr("Your branches slip out of the web.");
+                else
+                    mpr("Your branches shred the web that entangled you.");
+            }
+        }
+        // Fall through to dragon form to leave nets.
 
     case transformation::dragon:
         if (you.attribute[ATTR_HELD])
         {
-            trap_def *trap = trap_at(you.pos());
-            if (trap && trap->type == TRAP_WEB)
-            {
-                mpr("You shred the web into pieces!");
-                destroy_trap(you.pos());
-            }
             int net = get_trapping_net(you.pos());
             if (net != NON_ITEM)
             {
                 mpr("The net rips apart!");
                 destroy_item(net);
+                stop_being_held();
             }
-
-            stop_being_held();
         }
         break;
 
@@ -1901,6 +1912,9 @@ bool transform(int pow, transformation which_trans, bool involuntary,
 
     case transformation::shadow:
         drain_player(25, true, true);
+        if (you.duration[DUR_CORONA])
+            you.duration[DUR_CORONA] = 0;
+
         if (you.invisible())
             mpr("You fade into the shadows.");
         else
@@ -1919,14 +1933,17 @@ bool transform(int pow, transformation which_trans, bool involuntary,
     if (!form_keeps_mutations(which_trans))
         you.stop_directly_constricting_all(false);
 
-    // Stop being constricted if we are now too large.
-    if (you.is_directly_constricted())
+    // Stop being constricted if we are now too large, or are now immune.
+    if (you.get_constrict_type() == CONSTRICT_MELEE)
     {
         actor* const constrictor = actor_by_mid(you.constricted_by);
         ASSERT(constrictor);
 
-        if (you.body_size(PSIZE_BODY) > constrictor->body_size(PSIZE_BODY))
+        if (you.body_size(PSIZE_BODY) > constrictor->body_size(PSIZE_BODY)
+            || you.res_constrict())
+        {
             you.stop_being_constricted();
+        }
     }
 
 
@@ -1965,7 +1982,7 @@ bool transform(int pow, transformation which_trans, bool involuntary,
 
     // Update merfolk swimming for the form change.
     if (you.has_innate_mutation(MUT_MERTAIL))
-        merfolk_check_swimming(false);
+        merfolk_check_swimming(env.grid(you.pos()), false);
 
     // Update skill boosts for the current state of equipment melds
     // Must happen before the HP check!
@@ -2082,7 +2099,7 @@ void untransform(bool skip_move)
 
         // Update merfolk swimming for the form change.
         if (you.has_innate_mutation(MUT_MERTAIL))
-            merfolk_check_swimming(false);
+            merfolk_check_swimming(env.grid(you.pos()), false);
     }
 
 #ifdef USE_TILE
@@ -2113,7 +2130,7 @@ void untransform(bool skip_move)
     }
 
     // Stop being constricted if we are now too large.
-    if (you.is_directly_constricted())
+    if (you.get_constrict_type() == CONSTRICT_MELEE)
     {
         actor* const constrictor = actor_by_mid(you.constricted_by);
         if (you.body_size(PSIZE_BODY) > constrictor->body_size(PSIZE_BODY))
@@ -2140,9 +2157,10 @@ void emergency_untransform()
  * Idempotent, so can be called after position/transformation change without
  * redundantly checking conditions.
  *
- * @param stepped Whether the player is performing a normal walking move.
+ * @param old_grid The feature type that the player was previously on.
+ * @param stepped  Whether the player is performing a normal walking move.
  */
-void merfolk_check_swimming(bool stepped)
+void merfolk_check_swimming(dungeon_feature_type old_grid, bool stepped)
 {
     const dungeon_feature_type grid = env.grid(you.pos());
     if (you.ground_level()
@@ -2153,6 +2171,10 @@ void merfolk_check_swimming(bool stepped)
     }
     else
         merfolk_stop_swimming();
+
+    // Flying above water grants evasion, so redraw even if not transforming.
+    if (feat_is_water(grid) || feat_is_water(old_grid))
+        you.redraw_evasion = true;
 }
 
 void merfolk_start_swimming(bool stepped)
@@ -2169,10 +2191,13 @@ void merfolk_start_swimming(bool stepped)
         mpr("...but don't expect to remain undetected.");
 
     you.fishtail = true;
-    remove_one_equip(EQ_BOOTS);
     you.redraw_evasion = true;
 
-    ash_check_bondage();
+    if (!you.melded[EQ_BOOTS])
+    {
+        remove_one_equip(EQ_BOOTS);
+        ash_check_bondage();
+    }
 
 #ifdef USE_TILE
     init_player_doll();
@@ -2183,11 +2208,15 @@ void merfolk_stop_swimming()
 {
     if (!you.fishtail)
         return;
+
     you.fishtail = false;
-    unmeld_one_equip(EQ_BOOTS);
     you.redraw_evasion = true;
 
-    ash_check_bondage();
+    if (!_init_equipment_removal(you.form).count(EQ_BOOTS))
+    {
+        unmeld_one_equip(EQ_BOOTS);
+        ash_check_bondage();
+    }
 
 #ifdef USE_TILE
     init_player_doll();
