@@ -30,8 +30,13 @@ using std::vector;
 
 struct wm_keyboard_event;
 
+#define UI_SCROLLER_SHADE_SIZE 12
+
 namespace ui {
 
+command_type get_action(int keyin);
+bool key_exits_popup(int keyin, bool extended=false);
+
 struct SizeReq
 {
     int min, nat;
