@@ -35,6 +35,8 @@ static inline void scaled_delay(unsigned int ms)
     delay(ms * Options.view_delay / DEFAULT_VIEW_DELAY);
 }
 
+void animation_delay(unsigned int ms, bool do_refresh);
+
 // beware, flash_view is broken for USE_TILE_LOCAL
 void flash_view(use_animation_type a, colour_t colour,
                 targeter *where = nullptr);
@@ -82,9 +84,8 @@ public:
 #ifdef USE_TILE
 void view_add_tile_overlay(const coord_def &gc, tileidx_t tile);
 #endif
-#ifndef USE_TILE_LOCAL
 void view_add_glyph_overlay(const coord_def &gc, cglyph_t glyph);
-#endif
+
 void view_clear_overlays();
 
 void run_animation(animation_type anim, use_animation_type type,
