@@ -19,7 +19,6 @@
 #include "libutil.h"
 #include "macro.h"
 #include "message.h"
-#include "misc.h" // frombool
 #include "mutation.h"
 #include "output.h"
 #include "playable.h"
@@ -34,11 +33,12 @@
 #include "status.h"
 #include "stringutil.h"
 #include "tag-version.h"
-#include "timed-effects.h" // zot clock
 #include "transform.h"
+#include "ui.h"
 #include "unicode.h"
 #include "view.h"
 #include "xom.h"
+#include "zot.h" // zot clock
 
 #ifdef WIZARD
 
@@ -94,6 +94,9 @@ void wizard_suppress()
 {
     you.wizard = false;
     you.suppress_wizard = true;
+#ifdef USE_TILE_LOCAL
+    tiles.layout_statcol();
+#endif
     redraw_screen();
     update_screen();
 }
@@ -117,9 +120,9 @@ void wizard_change_species()
         return;
     }
 
-    const species_type sp = find_species_from_string(specs);
+    const species_type sp = species::from_str_loose(specs);
 
-    // Means find_species_from_string couldn't interpret `specs`.
+    // Means from_str_loose couldn't interpret `specs`.
     if (sp == SP_UNKNOWN)
     {
         mpr("That species isn't available.");
@@ -209,8 +212,15 @@ void wizard_heal(bool super_heal)
         you.duration[DUR_WEAK] = 0;
         you.duration[DUR_NO_HOP] = 0;
         you.duration[DUR_LOCKED_DOWN] = 0;
-        you.props["corrosion_amount"] = 0;
+        you.duration[DUR_NO_MOMENTUM] = 0;
+        you.props[CORROSION_KEY] = 0;
+        you.duration[DUR_BARBS] = 0;
+        you.attribute[ATTR_BARBS_POW] = 0;
+        you.props.erase(BARBS_MOVE_KEY);
+        you.duration[DUR_SICKNESS]  = 0;
+        you.duration[DUR_EXHAUSTED] = 0;
         you.duration[DUR_BREATH_WEAPON] = 0;
+        you.duration[DUR_BLINKBOLT_COOLDOWN] = 0;
         delete_all_temp_mutations("Super heal");
         you.stat_loss.init(0);
         you.attribute[ATTR_STAT_LOSS_XP] = 0;
@@ -220,11 +230,9 @@ void wizard_heal(bool super_heal)
     else
         mpr("Healing.");
 
-    // Clear most status ailments.
-    you.disease = 0;
+    // Clear some status ailments.
     you.duration[DUR_CONF]      = 0;
     you.duration[DUR_POISONING] = 0;
-    you.duration[DUR_EXHAUSTED] = 0;
     set_hp(you.hp_max);
     set_mp(you.max_magic_points);
     you.redraw_hit_points = true;
@@ -360,6 +368,8 @@ void wizard_exercise_skill()
 
     if (skill == SK_NONE)
         mpr("That skill doesn't seem to exist.");
+    else if (is_removed_skill(skill))
+        mpr("That skill was removed.");
     else
     {
         mpr("Exercising...");
@@ -372,14 +382,28 @@ void wizard_set_skill_level(skill_type skill)
     if (skill == SK_NONE)
         skill = debug_prompt_for_skill("Which skill (by name)? ");
 
+    if (is_removed_skill(skill)){
+        mpr("That skill was removed.");
+        return;
+    }
+
     if (skill == SK_NONE)
     {
         mpr("That skill doesn't seem to exist.");
         return;
     }
 
+    if (is_useless_skill(skill))
+    {
+        mpr("Can't change a useless skill.");
+        return;
+    }
+
     mpr(skill_name(skill));
-    double amount = prompt_for_float("To what level? ");
+    const double old_amount = you.skill(skill, 10, true) * 0.1;
+    string prompt = make_stringf("To what level? (current = %.1f) ",
+                                 old_amount);
+    double amount = prompt_for_float(prompt.c_str());
 
     if (amount < 0 || amount > 27)
     {
@@ -387,8 +411,6 @@ void wizard_set_skill_level(skill_type skill)
         return;
     }
 
-    const int old_amount = you.skills[skill];
-
     set_skill_level(skill, amount);
 
     if (amount == 27)
@@ -561,8 +583,7 @@ void wizard_set_stats()
     you.base_stats[STAT_DEX] = debug_cap_stat(sdex);
     you.stat_loss.init(0);
     you.attribute[ATTR_STAT_LOSS_XP] = 0;
-    you.redraw_stats.init(true);
-    you.redraw_evasion = true;
+    notify_stat_change();
 }
 
 void wizard_edit_durations()
@@ -715,7 +736,7 @@ static void reset_ds_muts_from_schedule(int xl)
                 // there. delete_mutation won't delete mutations otherwise.
                 // This step doesn't affect temporary mutations.
                 you.innate_mutation[mut]--;
-                delete_mutation(mut, "level change", false, true, false, false);
+                delete_mutation(mut, "level change", false, true, false);
             }
             if (you.innate_mutation[mut] < innate_levels)
                 perma_mutate(mut, innate_levels - you.innate_mutation[mut], "level change");
@@ -822,14 +843,26 @@ void wizard_get_god_gift()
         return;
     }
 
+    if (you_worship(GOD_ASHENZARI))
+    {
+        ashenzari_offer_new_curse();
+        return;
+    }
+
+    if (you_worship(GOD_YREDELEMNUL))
+    {
+        give_yred_bonus_zombies(min(piety_rank() + 1, NUM_PIETY_STARS));
+        return;
+    }
+
     if (!do_god_gift(true))
         mpr("Nothing happens.");
 }
 
 void wizard_toggle_xray_vision()
 {
-    you.xray_vision = !you.xray_vision;
-    mprf("X-ray vision %s.", you.xray_vision ? "enabled" : "disabled");
+    you.wizard_vision = !you.wizard_vision;
+    mprf("X-ray vision %s.", you.wizard_vision ? "enabled" : "disabled");
     viewwindow(true);
     update_screen();
 }
@@ -930,9 +963,9 @@ void wizard_transform()
 
 void wizard_join_religion()
 {
-    if (you.species == SP_DEMIGOD)
+    if (you.has_mutation(MUT_FORLORN))
     {
-        mpr("Not even in wizmode may Demigods worship a god!");
+        mpr("Not even in wizmode may divine creatures worship a god!");
         return;
     }
     god_type god = choose_god();
@@ -967,8 +1000,8 @@ void wizard_xom_acts()
 
     if (specs[0] == '\0')
     {
-        const maybe_bool nice = you_worship(GOD_XOM) ? MB_MAYBE :
-                                frombool(coinflip());
+        const maybe_bool nice = you_worship(GOD_XOM) ? maybe_bool::maybe :
+                                coinflip();
         const xom_event_type result = xom_acts(severity, nice);
         dprf("Xom did '%s'.", xom_effect_to_name(result).c_str());
 #ifndef DEBUG_DIAGNOSTICS
